package com.hyc.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hyc.domain.ConnectionParameter;
import com.hyc.domain.SummarizeResp;
import com.hyc.result.Result;
import com.hyc.service.SummarizeService;
import com.hyc.util.CalculateCheckDigit;
import com.hyc.util.ConversionUtil;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * 创建Mqtt客户端
 *
 * @author YouChe·He
 * @ClassName: CreateMqttClient
 * @Description: 创建Mqtt客户端
 * @CreateTime: 2024/5/31 10:25
 */
@Slf4j
@RestController
@RequestMapping("/create")
public class SummarizeController {

    @Autowired
    private SummarizeService summarizeService;

    @PostMapping("/createMqttClient")
    public String createMqttClient(@RequestBody List<ConnectionParameter> connectionParameterList){

        log.error("进来了!!!");
        for (ConnectionParameter connectionParameter : connectionParameterList) {
            boolean oneMqttClient = createOneMqttClient(connectionParameter);
            if (oneMqttClient==true){
                log.warn("broker为:{}的mqtt客户端连接成功",connectionParameter.getBroker());
            }
        }

        return "成功!";
    }

    @GetMapping("/getAllLoadInfo")
    public Result<SummarizeResp> getAllLoadInfo(){
        return summarizeService.getAllLoadInfo();
    }



    public boolean createOneMqttClient(ConnectionParameter connectionParameter){

        int qos = 0;
        try {
            MqttClient client = null;

            client = new MqttClient(connectionParameter.getBroker(), connectionParameter.getClientid(), new MemoryPersistence());


            // 连接参数
            MqttConnectOptions options = new MqttConnectOptions();
            if (connectionParameter.getUsername() !=null && !"".equals(connectionParameter.getUsername())){
                options.setUserName(connectionParameter.getUsername());
                options.setPassword(connectionParameter.getPassword().toCharArray());
            }

            options.setConnectionTimeout(60);
            options.setKeepAliveInterval(60);
            // 设置回调
            client.setCallback(new MqttCallback() {

                @Override
                public void connectionLost(Throwable cause) {
                    System.out.println("connectionLost: " + cause.getMessage());
                }

                @Override
                public void messageArrived(String topic, MqttMessage message) {
                    System.out.println("topic: " + topic);
                    System.out.println("Qos: " + message.getQos());
                    System.out.println("message content: " + new String(message.getPayload()));
                    String resultString = ConversionUtil.hexStringToString(new String(message.getPayload()));
                    log.warn("解析后的字符串是:{}",resultString);
                    log.warn("长度为:{}",resultString.length());

                    int count =0;
                    String realString = resultString.substring(1, 207);

                    int[] intArr ={17,13,11,10,6,11,6,5,9,1,
                            2,2,5,6,5,4,6,5,8,6,
                            6,6,2,5,6,4,4,6,6,6,
                            1,1,1,1,1,1,1,1,1,1,
                            1,1,1,1,1,1,1};
                    String[] strArr = {"vin","startTime","longitude","latitude","speed","mileage","voltage","current",
                            "resistance","gear","accelerationPedal","brakePedal",
                            "fuelConsumptionRate","motorControllerTemperature","motorSpeed","motorTorque",
                            "motorTemperature","motorVoltage","motorCurrent","remainingBattery","maximumFeedbackPower","maximumDischargePower",
                            "selfCheckCounter","totalBatteryCurrent","totalBatteryVoltage","singleBatteryMaxVoltage","singleBatteryMinVoltage",
                            "singleBatteryMaxTemperature","singleBatteryMinTemperature","availableBatteryCapacity","vehicleStatus",
                            "chargingStatus","operatingStatus","socStatus","chargingEnergyStorageStatus","driveMotorStatus",
                            "positionStatus","easStatus","ptcStatus","epsStatus","absStatus","mcuStatus",
                            "heatingStatus","batteryStatus","batteryInsulationStatus","dcdcStatus","chgStatus"};
                    LinkedHashMap<String, Object> linkedHashMap = new LinkedHashMap<>();
                    for (int i = 0; i < 47; i++) {
                        String substring = realString.substring(count, count + intArr[i]);
                        linkedHashMap.put(strArr[i],substring);
                        count = count + intArr[i];
                    }
                    log.warn("hashMap:{}",linkedHashMap);
                    ObjectMapper objectMapper = new ObjectMapper();
                    try {
                        String json = objectMapper.writeValueAsString(linkedHashMap);
                        log.error("json格式:{}",json);
                    } catch (JsonProcessingException e) {
                        e.printStackTrace();
                    }

                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken token) {
                    System.out.println("deliveryComplete---------" + token.isComplete());
                }

            });
            client.connect(options);
            client.subscribe(connectionParameter.getTopic(), qos);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }


}
