package com.hyc.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.DescribeInstancesRequest;
import com.aliyun.ecs20140526.models.DescribeInstancesResponse;
import com.aliyun.teaopenapi.models.Config;
import com.aliyun.teautil.models.RuntimeOptions;
import com.hyc.domain.GetWayServerLoad;
import com.hyc.domain.SummarizeResp;
import com.hyc.domain.TimeSortCar;
import com.hyc.domain.VinIp;
import com.hyc.mapper.SummarizeMapper;
import com.hyc.result.Result;
import com.hyc.service.SummarizeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 实现
 *
 * @author YouChe·He
 * @ClassName: SummarizeServiceImpl
 * @Description: 实现
 * @CreateTime: 2024/6/2 16:46
 */
@Service
@Slf4j
public class SummarizeServiceImpl implements SummarizeService {

    @Autowired
    private SummarizeMapper summarizeMapper;
    @Autowired
    private RestTemplate restTemplate;
    @Override
    public Result<SummarizeResp> getAllLoadInfo() {

        ArrayList<GetWayServerLoad> getWayServerLoads = new ArrayList<>();

        Integer carSum = 0;

        Double allLoad = 0.0;

        Integer gateWayServerNum = 0;

        Integer dataAnalyze = 22;



        ArrayList<String> ipList = new ArrayList<>();

        List<VinIp> connectLongVinList = summarizeMapper.getAllLoadInfo();

        List<VinIp> newConnectVinList = summarizeMapper.getNewConnectVin();


        try {
            Client client = this.createClient();
            //填写地区参数和实例规格参数
            DescribeInstancesRequest describeInstancesRequest = new DescribeInstancesRequest()
                    .setRegionId("cn-shanghai")
                    .setInstanceType("ecs.e-c1m2.xlarge");
            RuntimeOptions runtime = new RuntimeOptions();

            //获得ip并添加进入集合
            DescribeInstancesResponse describeInstancesResponse = client.describeInstancesWithOptions(describeInstancesRequest, runtime);
            List<List<String>> ipListList = describeInstancesResponse.getBody().instances.getInstance().stream().map(instance -> instance.publicIpAddress.ipAddress).collect(Collectors.toList());
            for (List<String> strings : ipListList) {
                for (String ip : strings) {
                    ipList.add(ip);
                    gateWayServerNum ++;
                }
                System.out.println("------------------------");
            }
            //遍历ip 访问URL
            for (String ip : ipList) {
                String url = "http://" + ip + ":8080/public/login";
                Map<String, Object> request = new HashMap<>();
                request.put("username", "fluxmq");
                request.put("password", "fluxmq");
                HttpHeaders httpHeaders = new HttpHeaders();
                httpHeaders.setContentType(MediaType.APPLICATION_JSON);
                HttpEntity<Map<String, Object>> r = new HttpEntity<>(request, httpHeaders);
                String result = restTemplate.postForObject(url, r, String.class);


                //http://fluxmq.muyu.icu/public/cluster

                int nextInt = new Random().nextInt(1000);
                String getInfoUrl = "http://" + ip + ":8080/public/cluster?random=" + nextInt;

                HttpHeaders httpHeadersGetInfo = new HttpHeaders();
                httpHeadersGetInfo.setContentType(MediaType.APPLICATION_JSON);
                httpHeadersGetInfo.setAccept(Collections.singletonList(MediaType.APPLICATION_JSON));
                httpHeadersGetInfo.set("Cookie", result);
                HttpEntity getInfoRequest = new HttpEntity(httpHeadersGetInfo);
                ResponseEntity<String> responseInfo = restTemplate.exchange(getInfoUrl, HttpMethod.GET, getInfoRequest, String.class, 1);

                log.info("响应是:{}", responseInfo.getBody());

                JSONArray jsonArray = JSON.parseArray(responseInfo.getBody());
                if (jsonArray.size() > 0) {

                    JSONObject jsonObject = jsonArray.getJSONObject(0);

                    Integer connectSize = Integer.valueOf(jsonObject.getJSONObject("mqttInfo").getString("connectSize"));
                    carSum = connectSize + carSum;
                    log.info("负载:{}",connectSize*1.0/100);

                    allLoad = connectSize*1.0/100;



                    String cSys = jsonObject.getJSONObject("cpuInfo").getString("idle");
                    int i = cSys.indexOf("%");
                    Double cpuInfo = Double.valueOf(cSys.substring(0, i));
                    log.info("CPU利用率:{}",cSys);

                    String heapUsed =jsonObject.getJSONObject("jvmInfo").getString("heap-used");
                    int x = heapUsed.indexOf("M");
                    String realHeapUsed = heapUsed.substring(0, x);

                    String noHeapUsed = jsonObject.getJSONObject("jvmInfo").getString("no_heap-used");
                    int y = noHeapUsed.indexOf("M");
                    String realNoHeapUsed = noHeapUsed.substring(0, y);

                    String heapMax = jsonObject.getJSONObject("jvmInfo").getString("heap-max");
                    int z = heapMax.indexOf("G");
                    String realHeapMax = heapMax.substring(0, z);

                    double v = (Double.valueOf(realHeapUsed) + Double.valueOf(realNoHeapUsed))/ (Double.valueOf(realHeapMax) * 1000) * 100;

                    BigDecimal bd = new BigDecimal(v).setScale(2, BigDecimal.ROUND_HALF_UP);
                    double roundedNumber = bd.doubleValue();
                    log.info("heapUsed:{},noHeapUsed:{},heapMax:{},内存使用率:{}",heapUsed,noHeapUsed,heapMax,roundedNumber);
                    getWayServerLoads.add(new GetWayServerLoad(ip,connectSize*1.0/100*100,roundedNumber,cpuInfo));
                    log.info("链接数量:{}", connectSize);
                } else {
                    log.error("得到的相应数据为null");
                }
            }

        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        ArrayList<TimeSortCar> sortCarArrayList = new ArrayList<>();

        for (VinIp vinIp : connectLongVinList) {
            Date connectTime = vinIp.getConnectTime();

            Date currentTime = new Date();

            // 计算时间差（单位：毫秒）
            long timeDifference = currentTime.getTime() - connectTime.getTime();

            // 转换为分钟数
            long minutesDifference = timeDifference / (1000 * 60);
            sortCarArrayList.add(new TimeSortCar(vinIp.getVin(),vinIp.getIp(),vinIp.getConnectTime(),minutesDifference));

        }


        Collections.sort(sortCarArrayList);
        SummarizeResp summarizeResp = new SummarizeResp(gateWayServerNum,dataAnalyze,allLoad / gateWayServerNum * 100,carSum,sortCarArrayList,getWayServerLoads,newConnectVinList);

        return Result.success(summarizeResp);
    }

    public  Client createClient() throws Exception {
        // 工程代码泄露可能会导致 AccessKey 泄露，并威胁账号下所有资源的安全性。以下代码示例仅供参考。
        // 建议使用更安全的 STS 方式，更多鉴权访问方式请参见：https://help.aliyun.com/document_detail/378657.html。
        Config config = new Config()
                // 必填，请确保代码运行环境设置了环境变量 ALIBABA_CLOUD_ACCESS_KEY_ID。
                .setAccessKeyId("LTAI5tAEQA9AgnqasQ7Y56cJ")
                // 必填，请确保代码运行环境设置了环境变量 ALIBABA_CLOUD_ACCESS_KEY_SECRET。
                .setAccessKeySecret("IsrnZ6dKBgEit5HXv2xyfo0xT8VGkj");
        // Endpoint 请参考 https://api.aliyun.com/product/Ecs
        config.endpoint = "ecs.cn-shanghai.aliyuncs.com";
        return new Client(config);
    }
}
